@@ -18,16 +18,16 @@
 #' subspace can only be spanned by at most \code{K-1} orthogonal vectors.
 #'
 #'
-#' @param X an \code{(n-by-p)} matrix or data frame whose rows are observations
+#' @param X an \eqn{(n\times p)} matrix or data frame whose rows are observations
 #' and columns represent independent variables.
-#' @param label a length-\code{n} vector of data class labels.
+#' @param label a length-\eqn{n} vector of data class labels.
 #' @param ndim an integer-valued target dimension.
 #'
 #' @return a named list containing
 #' \describe{
-#' \item{Y}{an \code{(n-by-ndim)} matrix whose rows are embedded observations.}
+#' \item{Y}{an \eqn{(n\times ndim)} matrix whose rows are embedded observations.}
 #' \item{trfinfo}{a list containing information for out-of-sample prediction.}
-#' \item{projection}{a \code{(p-by-ndim)} whose columns are basis for projection.}
+#' \item{projection}{a \eqn{(p\times ndim)} whose columns are basis for projection.}
 #' }
 #'
 #' @examples
@@ -73,6 +73,10 @@ do.lda <- function(X, label, ndim=2){
   if (K==n){
     warning("* do.lda : given 'label' has all unique elements.")
   }
+  if (any(is.na(label))||(any(is.infinite(label)))){
+    warning("* Supervised Learning : any element of 'label' as NA or Inf will simply be considered as a class, not missing entries.")
+  }
+
   #   3. ndim
   if (!check_ndim(ndim,p)){
     stop("* do.lda : 'ndim' is a positive integer in [1,#(covariates)].")
