@@ -1,10 +1,10 @@
-#' Locality Preserving Projections
+#' Locality Preserving Projection
 #'
 #' \code{do.lpp} is a linear approximation to Laplacian Eigenmaps. More precisely,
 #' it aims at finding a linear approximation to the eigenfunctions of the Laplace-Beltrami
 #' operator on the graph-approximated data manifold.
 #'
-#' @param X an \code{(n-by-p)} matrix or data frame whose rows are observations
+#' @param X an \eqn{(n\times p)} matrix or data frame whose rows are observations
 #' @param ndim an integer-valued target dimension.
 #' @param type a vector of neighborhood graph construction. Following types are supported;
 #'  \code{c("knn",k)}, \code{c("enn",radius)}, and \code{c("proportion",ratio)}.
@@ -13,14 +13,16 @@
 #' @param symmetric one of \code{"intersect"}, \code{"union"} or \code{"asymmetric"} is supported. Default is \code{"union"}.
 #' See also \code{\link{aux.graphnbd}} for more details.
 #' @param weight \code{TRUE} to perform LPP on weighted graph, or \code{FALSE} otherwise.
-#' @param preprocess an additional option for preprocessing the data.
+#' @param preprocess  an additional option for preprocessing the data.
+#' Default is "center" and other options of "decorrelate" and "whiten"
+#' are supported. See also \code{\link{aux.preprocess}} for more details.
 #' @param t bandwidth for heat kernel in \eqn{(0,\infty)}
 #'
 #' @return a named list containing
 #' \describe{
-#' \item{Y}{an \code{(n-by-ndim)} matrix whose rows are embedded observations.}
+#' \item{Y}{an \eqn{(n\times ndim)} matrix whose rows are embedded observations.}
 #' \item{eigval}{a vector of eigenvalues corresponding to basis expansion in an ascending order.}
-#' \item{projection}{a \code{(p-by-ndim)} whose columns are basis for projection.}
+#' \item{projection}{a \eqn{(p\times ndim)} whose columns are basis for projection.}
 #' \item{trfinfo}{a list containing information for out-of-sample prediction.}
 #' }
 #'
@@ -55,7 +57,7 @@
 #' @author Kisung You
 #' @rdname linear_LPP
 #' @export
-do.lpp <- function(X,ndim=2,type=c("proportion",0.1),symmetric="union",weight=TRUE,preprocess="null",t=1.0){
+do.lpp <- function(X,ndim=2,type=c("proportion",0.1),symmetric="union",weight=TRUE,preprocess="center",t=1.0){
   # 1. typecheck is always first step to perform.
   aux.typecheck(X)
   if ((!is.numeric(ndim))||(ndim<1)||(ndim>ncol(X))||is.infinite(ndim)||is.na(ndim)){
@@ -82,8 +84,8 @@ do.lpp <- function(X,ndim=2,type=c("proportion",0.1),symmetric="union",weight=TR
     stop("* do.lpp : 'weight' should be a logical variable.")
   }
   algpreprocess = preprocess
-  if (!is.element(algpreprocess,c("null","center","whiten","decorrelate"))){
-    stop("* do.lpp : 'preprocess' should be one of 4 values.")
+  if (!is.element(algpreprocess,c("center","whiten","decorrelate"))){
+    stop("* do.lpp : 'preprocess' should be one of 3 options.")
   }
   if (!is.numeric(t)||(t<=0)||is.na(t)||is.infinite(t)){
     stop("* do.lpp : 't' is a bandwidth parameter in (0,infinity).")
@@ -134,10 +136,13 @@ do.lpp <- function(X,ndim=2,type=c("proportion",0.1),symmetric="union",weight=TR
     return(result)
   } else {
     # 7. return output
+    #   1. adjust projection
+    projection = aux.adjprojection(eigvecs[,1:ndim])
+    #   2. return output
     result = list()
-    result$Y = pX %*% eigvecs[,1:ndim]
+    result$Y = pX %*% projection
     result$eigval = eigvals[1:ndim]
-    result$projection = eigvecs[,1:ndim]
+    result$projection = projection
     trfinfo$algtype = "linear"
     result$trfinfo = trfinfo
     return(result)
